@@ -10,6 +10,7 @@ import logging
 import pandas as pd
 import numpy as np
 import re
+from dateutil.relativedelta import relativedelta
 
 from beancount.core.number import Decimal
 from beancount.core import data
@@ -17,18 +18,30 @@ from beancount.core import prices
 from beancount.core import convert
 from beancount.core import inventory
 from beancount.core import account_types
+from beancount.core import amount
 from beancount.query import query
 from beancount.core.data import Custom
 from beancount.parser import options
 
 class BeancountEnvelope:
 
-    def __init__(self, entries, options_map):
+    def __init__(self, entries, options_map,currency):
 
         self.entries = entries
         self.options_map = options_map
-        self.currency = self._find_currency(options_map)
-        self.start_date, self.budget_accounts, self.mappings = self._find_envelop_settings()
+        self.currency = currency
+        self.negative_rollover = False
+        self.months_ahead = 0
+
+        if self.currency:
+            self.etype="envelope"+self.currency
+        else:
+            self.etype="envelope"
+
+        self.start_date, self.budget_accounts, self.mappings, self.income_accounts, self.months_ahead = self._find_envelop_settings()
+
+        if not self.currency:
+            self.currency=self._find_currency(options_map)
 
         decimal_precison = '0.00'
         self.Q = Decimal(decimal_precison)
@@ -41,7 +54,7 @@ class BeancountEnvelope:
         # TODO should be able to assert errors
 
         # Compute end of period
-        self.date_end = datetime.date(today.year, today.month, today.day)
+        self.date_end = datetime.date(today.year, today.month, today.day) + relativedelta(months=+self.months_ahead)
 
         self.price_map = prices.build_price_map(entries)
         self.acctypes = options.get_account_types(options_map)
@@ -62,9 +75,11 @@ class BeancountEnvelope:
         start_date = None
         budget_accounts= []
         mappings = []
+        income_accounts = []
+        months_ahead = 0
 
         for e in self.entries:
-            if isinstance(e, Custom) and e.type == "envelope":
+            if isinstance(e, Custom) and e.type == self.etype:
                 if e.values[0].value == "start date":
                     start_date = e.values[1].value
                 if e.values[0].value == "budget account":
@@ -75,7 +90,16 @@ class BeancountEnvelope:
                         e.values[2].value
                     )
                     mappings.append(map_set)
-        return start_date, budget_accounts, mappings
+                if e.values[0].value == "income account":
+                    income_accounts.append(re.compile(e.values[1].value))
+                if e.values[0].value == "currency":
+                    self.currency = e.values[1].value
+                if e.values[0].value == "negative rollover":
+                    if e.values[1].value == "allow":
+                        self.negative_rollover = True
+                if e.values[0].value == "months ahead":
+                    months_ahead = int(e.values[1].value)
+        return start_date, budget_accounts, mappings, income_accounts, months_ahead
 
     def envelope_tables(self):
 
@@ -119,7 +143,7 @@ class BeancountEnvelope:
                     row[month, 'available'] = row[month, 'budgeted'] + row[month, 'activity']
                 else:
                     prev_available = row[months[index2-1],'available']
-                    if prev_available > 0:
+                    if prev_available > 0 or self.negative_rollover:
                         row[month, 'available'] = prev_available + row[month, 'budgeted'] + row[month, 'activity']
                     else:
                         row[month, 'available'] = row[month, 'budgeted'] + row[month, 'activity']
@@ -168,7 +192,7 @@ class BeancountEnvelope:
         for index, month in enumerate(months):
             self.income_df.loc["To Be Budgeted", month] = Decimal(self.income_df[month].sum())
 
-        return self.income_df, self.envelope_df
+        return self.income_df, self.envelope_df, self.currency
 
     def _calculate_budget_activity(self):
 
@@ -207,9 +231,15 @@ class BeancountEnvelope:
 
                 account_type = account_types.get_account_type(account)
                 if posting.units.currency != self.currency:
-                    continue
+                    orig=posting.units.number
+                    if posting.price is not None:
+                        converted=posting.price.number*orig
+                        posting=data.Posting(posting.account,amount.Amount(converted,self.currency), posting.cost, None, posting.flag,posting.meta)
+                    else:
+                        continue
 
-                if account_type == self.acctypes.income:
+                if (account_type == self.acctypes.income
+                    or (any(regexp.match(account) for regexp in self.income_accounts))):
                     account = "Income"
                 elif any(regexp.match(posting.account) for regexp in self.budget_accounts):
                     continue
@@ -258,7 +288,7 @@ class BeancountEnvelope:
     def _calc_budget_budgeted(self):
         rows = {}
         for e in self.entries:
-            if isinstance(e, Custom) and e.type == "envelope":
+            if isinstance(e, Custom) and e.type == self.etype:
                 if e.values[0].value == "allocate":
                     month = f"{e.date.year}-{e.date.month:02}"
                     self.envelope_df.loc[e.values[1].value,(month,'budgeted')] = Decimal(e.values[2].value)
